#!/bin/sh

PATH=/var/epages:$PATH

. functions.sh
. $(etc_default)

BN=${0##*/}

##############################################################################
# arguments                                                                  #
##############################################################################

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $1"
  echo "Usage:"
  echo "  $BN -p PATCH_VERSION [-i INITIAL_VERSION]"
  echo "Options:"
  echo "  -p : ePages target version (X.Y.Z as 6.15.3)"
  echo "  -i : ePages initial version (X.Y.Z as 6.15.2)"
  echo "Example:"
  echo "  $BN -i 6.15.2 -p 6.15.3"
  exit 1
}

unset INITIAL_VERSION PATCH_VERSION

while getopts i:p: c ; do
  case $c in
    i) INITIAL_VERSION=$OPTARG ;;
    p) PATCH_VERSION=$OPTARG ;;
    *) usage ;;
  esac
done

[[ -z $PATCH_VERSION ]] && usage "$BN: Option -p missing"
[[ -n ${PATCH_VERSION//[0-9.]/} ]] && usage "$BN: Option -p: void format"
PATCH=(${PATCH_VERSION//./ })
[[ ${#PATCH[@]} = 3 ]] || usage "$BN: Option -p: void format"

if [[ -n $INITIAL_VERSION ]] ; then
  [[ -n ${INITIAL_VERSION//[0-9.]/} ]] && usage "$BN: Option -p: void format"
  INITIAL=(${INITIAL_VERSION//./ })
  [[ ${#INITIAL[@]} = 3 ]] || usage "$BN: Option -p: void format"
  
fi

# check if epages-arch-i386 exists (since 6.14.4)

if [[ -f /etc/yum.repos.d/epages-arch.repo ]] ; then
  grep -q '\[epages-arch-i386\]' /etc/yum.repos.d/epages-arch.repo || {
    unset arch
    while read i ; do
      if [[ $i = "[epages-arch]" ]] ; then
        arch="[epages-arch-i386]"
      elif [[ -n $arch && ${i#\[} != $i ]] ; then
        echo -e "$arch" ; unset arch
      elif [[ -n $arch ]] ; then
        i=${i/x86_64/i386}
        [[ ${i#enabled} != $i ]] && arch="$arch\nenabled = 0" || arch="$arch\n$i"
      fi
      echo "$i"
    done < /etc/yum.repos.d/epages-arch.repo > /tmp/$$.pid
    [[ -n $arch ]] && echo -e "\n$arch" >> /tmp/$$.pid
    cat /tmp/$$.pid > /etc/yum.repos.d/epages-arch.repo
    rm -f /tmp/$$.pid
  }
fi

started_by_rpm $$ && exit 0

# third party capabilities

cap3rd=(graphviz nodejs mp3splt)

# are provided by third party repositories

if [[ -f /etc/SuSE-release ]] ; then
  repo3rd=("" nodejs-sles packman-multimedia)
else
  repo3rd=(graphviz-rhel "" atrpms)
fi
KERNEL=$(uname -r)
unset RELEASEVER
case $KERNEL in
  2.6.18-*) RELEASEVER=5 ;;
  2.6.32-*) RELEASEVER=6 ;;
esac

RET=0
for i in `seq 0 $((${#cap3rd[@]} -1))` ; do
  [[ -z ${repo3rd[$i]} ]] && continue
  [[ $KERNEL != ${KERNEL#2.6.18-} && ${repo3rd[$i]} = atrpms ]] && continue

  echo -n "testing RPM package: ${cap3rd[$i]} -> "
  if rpm --quiet -q ${cap3rd[$i]} ; then
    echo installed ; continue
  elif [[ ! -f /etc/yum.repos.d/${repo3rd[$i]}.repo ]] ; then
    echo "failure"
    echo -n "ERROR: " 1>&2
    ls /etc/yum.repos.d/${repo3rd[$i]}.repo
    echo "Did you run rpmify-epages.sh successfully before?:" 1>&2
    echo -e "curl http://epages.com/RPMS/rpmify/rpmify-epages.sh | sh\n" 1>&2
    RET=1 ; continue
  else
    BASEURL=$(awk -F= '/^baseurl=/{print $2; exit}' /etc/yum.repos.d/${repo3rd[$i]}.repo | sed "s,\$releasever,$RELEASEVER,;s,\$basearch,$EPAGES_ARCH,;s,[[:space:]],,g")
    if [[ -z $BASEURL ]] ; then
      echo "failure"
      echo "ERROR: /etc/yum.repos.d/${repo3rd[$i]}.repo: no baseurl= found" 1>&2
      echo "Did you run rpmify-epages.sh successfully before?:" 1>&2
      echo -e "curl http://epages.com/RPMS/rpmify/rpmify-epages.sh | sh\n" 1>&2
      RET=1 ; continue
    else
      echo $BASEURL
    fi
  fi

  if $PERL -MLWP::Simple -e "exit scalar head '$BASEURL'" ; then
    echo "ERROR: Cannot get RPM package '${cap3rd[$i]}' from '$BASEURL'" 1>&2
    echo -e "RPM package '${cap3rd[$i]}' must be installed before starting patch.\n" 1>&2
    RET=1
  fi
done

[[ $RET = 0 ]] || exit $RET

# epages update repository

REPO=/etc/yum.repos.d/epages-update-archive.repo

echo -n "testing repository: epages-update-$PATCH_VERSION -> "
if [[ ! -f $REPO ]] ; then
  if [[ -f $REPO.rpmsave ]] ; then
    cp -p $REPO.rpmsave $REPO
  else
    echo "failure"
    echo -n "ERROR: " 1>&2
    ls $REPO
    echo "Did you run rpmify-epages.sh successfully before?:" 1>&2
    echo -e "curl http://epages.com/RPMS/rpmify/rpmify-epages.sh | sh\n" 1>&2
    RET=2
  fi
fi

[[ $RET = 0 ]] || exit $RET

BASEURL=$(ini-config.sh -f $REPO -s epages-update-$PATCH_VERSION -k baseurl -G)

if [[ -z $BASEURL ]] ; then
  echo "failure"
  echo "ERROR: $REPO: no baseurl= found in section epages-update-$PATCH_VERSION" 1>&2
  echo "Did you run rpmify-epages.sh successfully before?:" 1>&2
  echo -e "curl http://epages.com/RPMS/rpmify/rpmify-epages.sh | sh\n" 1>&2
  RET=2
else
  BASEURL=${BASEURL%/}
  echo $BASEURL
fi

[[ $RET = 0 ]] || exit $RET

if [[ -n $INITIAL_VERSION ]] ; then
  # check if baseurl ends in /$PATCH_VERSION/6.15-update
  if [[ $BASEURL = ${BASEURL%/$PATCH_VERSION/${INITIAL[0]}.${INITIAL[1]}-update} ]] ; then
    echo "failure"
    echo "ERROR: $REPO: baseurl= in section epages-update-$PATCH_VERSION" 1>&2
    echo "does not end in '/$PATCH_VERSION/${INITIAL[0]}.${INITIAL[1]}-update'" 1>&2
    echo "Did you run rpmify-epages.sh successfully before?:" 1>&2
    echo -e "curl http://epages.com/RPMS/rpmify/rpmify-epages.sh | sh\n" 1>&2
    RET=2
  fi
fi

[[ $RET = 0 ]] || exit $RET

if $PERL -MLWP::Simple -e "exit scalar head '$BASEURL'" ; then
  echo "ERROR: $REPO: can't connect to baseurl=$BASEURL" 1>&2
  echo "Did you run rpmify-epages.sh successfully before?:" 1>&2
  echo -e "curl http://epages.com/RPMS/rpmify/rpmify-epages.sh | sh\n" 1>&2
  RET=2
fi

type -p yum >/dev/null 2>&1 && yum clean all || zypper clean -a

exit $RET
