use strict;
use Test::More tests => 152;

use DBI;
use Data::Dumper;
use File::Basename qw(dirname);
use File::Spec;
use Getopt::Long;

use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Core::API::File qw( RealPath GetFileContent RunOnDirectory);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw (SystemCommand);
use DE_EPAGES::Database::API::Connection qw (ConnectSite GetCurrentDBHandle);
use DE_EPAGES::Database::API::OrderFile qw( GetSqlFileOrder );

sub Prepare {
    my ($CurrentDir) = @_;

    my $PerlExe = $^O eq 'MSWin32' ? 'perl' : $ENV{PERL};

    RunOnDirectory( $CurrentDir, sub {
        #drop patch system info db
        SystemCommand([$PerlExe, 'Scripts/dropPatchDB.pl'], {'Print' => 1});

        #generate patch system info db
        SystemCommand([$PerlExe, 'Scripts/createPatchDB.pl'], {'Print' => 1});
    });

    #connect to patch system info db
    ConnectSite('PatchSystemInfoDB');
    my $dbh = GetCurrentDBHandle();
    $dbh->{RaiseError} = 1;

    #get sql files in right order
    my $aFileNames = GetSqlFileOrder("$CurrentDir/Database/mysql");

    #for each filename process sql statements
    foreach my $FileName (@$aFileNames){
        my $rContent = GetFileContent( $FileName );

        #foreach sql statement in file
        foreach my $SQLStatement ( grep { /\S/ } split /\ngo\b/i, $$rContent ) {
            print "$SQLStatement\n\n";
            #execute it
            $dbh->do($SQLStatement);
        }
    }

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('1','1.1.1')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('2','1.1.2')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('3','1.1.3')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('4','1.2.1')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('5','1.2.2')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('6','1.2.3')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('7','2.1.1')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('8','2.1.2')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('9','2.1.3')");
    $dbh->do("INSERT INTO patchsystemattribute (keyname,value) VALUES ('ePagesTargetVersion','2.1.3')");
}

sub Cleanup {
    my ($CurrentDir) = @_;

    my $PerlExe = $^O eq 'MSWin32' ? 'perl' : $ENV{PERL};
    RunOnDirectory( $CurrentDir, sub {
        #drop patch system info db
        SystemCommand([$PerlExe, 'Scripts/dropPatchDB.pl'], {'Print' => 1});
    });
}

sub Main {
    local $| = 1;

    my $Help;
    GetOptions( 'help' => \$Help );
    usage() if $Help;

    my $CurrentDir = dirname(dirname(File::Spec->rel2abs($0)));

    #test preperation
    Prepare($CurrentDir);

    #get patchsysteminfohandler module file name
    my $PatchSystemInfoHandlerModul = "$CurrentDir/API/Patch/PatchSystemInfoHandler.pm";
    my $PatchSystemInfoHandler;

    RunOnDirectory($CurrentDir, sub {
        #use file
        do $PatchSystemInfoHandlerModul; die $@ if $@;

        #test 01 - exists patch system info
        $PatchSystemInfoHandler = API::Patch::PatchSystemInfoHandler->new();
    });

    $PatchSystemInfoHandler->setStartTime();
    my $StartTime = $PatchSystemInfoHandler->getStartTime();

    ok($PatchSystemInfoHandler->existsBasicAttribute('ePagesTargetVersion'), 'key \'ePagesTargetVersion\' exists');

    ok(not ($PatchSystemInfoHandler->existsBasicAttribute('EpagesSharedDir')), 'key \'EpagesSharedDir\' does not exists');

    eval{
        ok(not $PatchSystemInfoHandler->existsBasicAttribute('AnyKey'), 'key \'AnyKey\' does not exists');
    };
    ok( ExistsError(), 'searching for key \'AnyKey\' throws error');


    is($PatchSystemInfoHandler->getBasicAttribute('ePagesTargetVersion'), '2.1.3', 'key \'ePagesTargetVersion\' is correct');

    ok($PatchSystemInfoHandler->getPatchTag() eq 'patch_2_1_3', 'patch tag \'patch_2_12_3\' is correct');

    $PatchSystemInfoHandler->setBasicAttribute('ePagesTargetVersion', '2.1.2');
    is($PatchSystemInfoHandler->getBasicAttribute('ePagesTargetVersion'), '2.1.2', 'key \'ePagesTargetVersion\' is correctly changed');

    is($PatchSystemInfoHandler->getPatchTag(), 'patch_2_1_2', 'patch tag \'patch_2_12_2\' is correct');


    my $TestPath = ['1.1.1','1.1.2','1.1.3','1.2.1','1.2.2','1.2.3','2.1.1','2.1.2','2.1.3'];


    is_deeply ($PatchSystemInfoHandler->validVersions, $TestPath, 'version path correct');

    ok ($PatchSystemInfoHandler->validVersion('1.2.2'), 'version \'1.2.2\' is valid');

    ok ( not ($PatchSystemInfoHandler->validVersion('5.5.5')), 'version \'5.5.5\' is not valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch'), 'function \'API::Patch::PatchPrepareHandler::doPatch\' is valid');

    ok ( not ($PatchSystemInfoHandler->validFunction('API::Patch::PatchPopelHandler::doPatch')), 'function \'API::Patch::PatchPopelHandler::doPatch\' is not valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_universal'), 'function \'API::Patch::PatchPrepareHandler::doPatch_universal\' is valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_1_2_2'), 'function \'API::Patch::PatchPrepareHandler::doPatch_1_2_2\' is valid');

    ok ( not ($PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_5_5_5')), 'function \'API::Patch::PatchPrepareHandler::doPatch_5_5_5\' is not valid');

    my $GroundFunctions = [
        'callSubHandlerWithVersionSources',
        'callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'doPatch',
        'doPatch_1_1_1',
        'doPatch_1_1_2',
        'doPatch_1_1_3',
        'doPatch_1_2_1',
        'doPatch_1_2_2',
        'doPatch_1_2_3',
        'doPatch_2_1_1',
        'doPatch_2_1_2',
        'doPatch_2_1_3',
        'doPatch_basic',
        'doPatch_checkPreCondition',
        'doPatch_universal',
        'toExecute',
        'useNow'
    ];

    is_deeply ( $PatchSystemInfoHandler->validGroundFunctions(), $GroundFunctions, 'ground functions correct');

    my $Functions = [
        'API::Patch::PatchPrepareHandler::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandler::doPatch',
        'API::Patch::PatchPrepareHandler::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandler::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandler::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandler::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandler::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandler::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandler::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandler::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandler::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandler::doPatch_basic',
        'API::Patch::PatchPrepareHandler::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandler::doPatch_universal',
        'API::Patch::PatchPrepareHandler::toExecute',
        'API::Patch::PatchPrepareHandler::useNow',
        'API::Patch::PatchDataDownloadHandler::callSubHandlerWithVersionSources',
        'API::Patch::PatchDataDownloadHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchDataDownloadHandler::doPatch',
        'API::Patch::PatchDataDownloadHandler::doPatch_1_1_1',
        'API::Patch::PatchDataDownloadHandler::doPatch_1_1_2',
        'API::Patch::PatchDataDownloadHandler::doPatch_1_1_3',
        'API::Patch::PatchDataDownloadHandler::doPatch_1_2_1',
        'API::Patch::PatchDataDownloadHandler::doPatch_1_2_2',
        'API::Patch::PatchDataDownloadHandler::doPatch_1_2_3',
        'API::Patch::PatchDataDownloadHandler::doPatch_2_1_1',
        'API::Patch::PatchDataDownloadHandler::doPatch_2_1_2',
        'API::Patch::PatchDataDownloadHandler::doPatch_2_1_3',
        'API::Patch::PatchDataDownloadHandler::doPatch_basic',
        'API::Patch::PatchDataDownloadHandler::doPatch_checkPreCondition',
        'API::Patch::PatchDataDownloadHandler::doPatch_universal',
        'API::Patch::PatchDataDownloadHandler::toExecute',
        'API::Patch::PatchDataDownloadHandler::useNow',
        'API::Patch::PatchPrepareHandlerVersionFolders::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerVersionFolders::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFolders::toExecute',
        'API::Patch::PatchPrepareHandlerVersionFolders::useNow',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::toExecute',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::useNow',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::toExecute',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::useNow',
        'API::Patch::PatchPrepareHandlerStoreTypes::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerStoreTypes::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_basic',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_universal',
        'API::Patch::PatchPrepareHandlerStoreTypes::toExecute',
        'API::Patch::PatchPrepareHandlerStoreTypes::useNow',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_basic',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_universal',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::toExecute',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms::useNow',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::toExecute',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::useNow',
        'API::Patch::PatchPrepareHandlerDojo::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerDojo::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerDojo::doPatch',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_basic',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerDojo::doPatch_universal',
        'API::Patch::PatchPrepareHandlerDojo::toExecute',
        'API::Patch::PatchPrepareHandlerDojo::useNow',
        'API::Patch::PatchPrepareHandlerJavaScript::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerJavaScript::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJavaScript::doPatch_universal',
        'API::Patch::PatchPrepareHandlerJavaScript::toExecute',
        'API::Patch::PatchPrepareHandlerJavaScript::useNow',
        'API::Patch::PatchPrepareHandlerCSS::callSubHandlerWithVersionSources',
        'API::Patch::PatchPrepareHandlerCSS::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPrepareHandlerCSS::doPatch',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_1_1_2',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_1_1_3',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_1_2_1',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_1_2_2',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_1_2_3',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_2_1_1',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_2_1_2',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_2_1_3',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_basic',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerCSS::doPatch_universal',
        'API::Patch::PatchPrepareHandlerCSS::toExecute',
        'API::Patch::PatchPrepareHandlerCSS::useNow',
        'API::Patch::PatchPreInstallHandlerWebServer::callSubHandlerWithVersionSources',
        'API::Patch::PatchPreInstallHandlerWebServer::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_1_1',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_1_2',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_1_3',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_2_1',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_2_2',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_2_3',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_2_1_1',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_2_1_2',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_2_1_3',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_basic',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_checkPreCondition',
        'API::Patch::PatchPreInstallHandlerWebServer::doPatch_universal',
        'API::Patch::PatchPreInstallHandlerWebServer::toExecute',
        'API::Patch::PatchPreInstallHandlerWebServer::useNow',
        'API::Patch::PatchPreInstallHandlerBackup::callSubHandlerWithVersionSources',
        'API::Patch::PatchPreInstallHandlerBackup::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_1_1',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_1_2',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_1_3',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_2_1',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_2_2',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_2_3',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_2_1_1',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_2_1_2',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_2_1_3',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_basic',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_checkPreCondition',
        'API::Patch::PatchPreInstallHandlerBackup::doPatch_universal',
        'API::Patch::PatchPreInstallHandlerBackup::toExecute',
        'API::Patch::PatchPreInstallHandlerBackup::useNow',
        'API::Patch::PatchPreInstallHandlerFiles::callSubHandlerWithVersionSources',
        'API::Patch::PatchPreInstallHandlerFiles::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_1_1',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_1_2',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_1_3',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_2_1',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_2_2',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_2_3',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_2_1_1',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_2_1_2',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_2_1_3',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_basic',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_checkPreCondition',
        'API::Patch::PatchPreInstallHandlerFiles::doPatch_universal',
        'API::Patch::PatchPreInstallHandlerFiles::toExecute',
        'API::Patch::PatchPreInstallHandlerFiles::useNow',
        'API::Patch::PatchInstallHandler::callSubHandlerWithVersionSources',
        'API::Patch::PatchInstallHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchInstallHandler::doPatch',
        'API::Patch::PatchInstallHandler::doPatch_1_1_1',
        'API::Patch::PatchInstallHandler::doPatch_1_1_2',
        'API::Patch::PatchInstallHandler::doPatch_1_1_3',
        'API::Patch::PatchInstallHandler::doPatch_1_2_1',
        'API::Patch::PatchInstallHandler::doPatch_1_2_2',
        'API::Patch::PatchInstallHandler::doPatch_1_2_3',
        'API::Patch::PatchInstallHandler::doPatch_2_1_1',
        'API::Patch::PatchInstallHandler::doPatch_2_1_2',
        'API::Patch::PatchInstallHandler::doPatch_2_1_3',
        'API::Patch::PatchInstallHandler::doPatch_basic',
        'API::Patch::PatchInstallHandler::doPatch_checkPreCondition',
        'API::Patch::PatchInstallHandler::doPatch_universal',
        'API::Patch::PatchInstallHandler::toExecute',
        'API::Patch::PatchInstallHandler::useNow',
        'API::Patch::PatchInstallHandlerCPAN::callSubHandlerWithVersionSources',
        'API::Patch::PatchInstallHandlerCPAN::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchInstallHandlerCPAN::doPatch',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_1_1_2',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_1_1_3',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_1_2_1',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_1_2_2',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_1_2_3',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_2_1_1',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_2_1_2',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_2_1_3',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_basic',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_checkPreCondition',
        'API::Patch::PatchInstallHandlerCPAN::doPatch_universal',
        'API::Patch::PatchInstallHandlerCPAN::toExecute',
        'API::Patch::PatchInstallHandlerCPAN::useNow',
        'API::Patch::PatchInstallHandlerSite::callSubHandlerWithVersionSources',
        'API::Patch::PatchInstallHandlerSite::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchInstallHandlerSite::doPatch',
        'API::Patch::PatchInstallHandlerSite::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerSite::doPatch_1_1_2',
        'API::Patch::PatchInstallHandlerSite::doPatch_1_1_3',
        'API::Patch::PatchInstallHandlerSite::doPatch_1_2_1',
        'API::Patch::PatchInstallHandlerSite::doPatch_1_2_2',
        'API::Patch::PatchInstallHandlerSite::doPatch_1_2_3',
        'API::Patch::PatchInstallHandlerSite::doPatch_2_1_1',
        'API::Patch::PatchInstallHandlerSite::doPatch_2_1_2',
        'API::Patch::PatchInstallHandlerSite::doPatch_2_1_3',
        'API::Patch::PatchInstallHandlerSite::doPatch_basic',
        'API::Patch::PatchInstallHandlerSite::doPatch_checkPreCondition',
        'API::Patch::PatchInstallHandlerSite::doPatch_universal',
        'API::Patch::PatchInstallHandlerSite::toExecute',
        'API::Patch::PatchInstallHandlerSite::useNow',
        'API::Patch::PatchInstallHandlerASPoolDB::callSubHandlerWithVersionSources',
        'API::Patch::PatchInstallHandlerASPoolDB::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_1_2',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_1_3',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_2_1',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_2_2',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_2_3',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_2_1_1',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_2_1_2',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_2_1_3',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_basic',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_checkPreCondition',
        'API::Patch::PatchInstallHandlerASPoolDB::doPatch_universal',
        'API::Patch::PatchInstallHandlerASPoolDB::toExecute',
        'API::Patch::PatchInstallHandlerASPoolDB::useNow',
        'API::Patch::PatchInstallHandlerStores::callSubHandlerWithVersionSources',
        'API::Patch::PatchInstallHandlerStores::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchInstallHandlerStores::doPatch',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_1_2',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_1_3',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_2_1',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_2_2',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_2_3',
        'API::Patch::PatchInstallHandlerStores::doPatch_2_1_1',
        'API::Patch::PatchInstallHandlerStores::doPatch_2_1_2',
        'API::Patch::PatchInstallHandlerStores::doPatch_2_1_3',
        'API::Patch::PatchInstallHandlerStores::doPatch_basic',
        'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition',
        'API::Patch::PatchInstallHandlerStores::doPatch_universal',
        'API::Patch::PatchInstallHandlerStores::toExecute',
        'API::Patch::PatchInstallHandlerStores::useNow',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::callSubHandlerWithVersionSources',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_1_2',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_1_3',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_2_1',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_2_2',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_2_3',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_2_1_1',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_2_1_2',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_2_1_3',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_basic',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_checkPreCondition',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_universal',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::toExecute',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs::useNow',
        'API::Patch::PatchInstallHandlerFlexShopTypes::callSubHandlerWithVersionSources',
        'API::Patch::PatchInstallHandlerFlexShopTypes::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_1_2',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_1_3',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_2_1',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_2_2',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_2_3',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_2_1_1',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_2_1_2',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_2_1_3',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_basic',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_checkPreCondition',
        'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_universal',
        'API::Patch::PatchInstallHandlerFlexShopTypes::toExecute',
        'API::Patch::PatchInstallHandlerFlexShopTypes::useNow',
        'API::Patch::PatchPostInstallHandler::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandler::doPatch',
        'API::Patch::PatchPostInstallHandler::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandler::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandler::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandler::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandler::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandler::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandler::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandler::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandler::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandler::doPatch_basic',
        'API::Patch::PatchPostInstallHandler::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandler::doPatch_universal',
        'API::Patch::PatchPostInstallHandler::toExecute',
        'API::Patch::PatchPostInstallHandler::useNow',
        'API::Patch::PatchPostInstallHandlerCleanUp::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandlerCleanUp::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_basic',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_universal',
        'API::Patch::PatchPostInstallHandlerCleanUp::toExecute',
        'API::Patch::PatchPostInstallHandlerCleanUp::useNow',
        'API::Patch::PatchPostInstallHandlerWebroot::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandlerWebroot::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_basic',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandlerWebroot::doPatch_universal',
        'API::Patch::PatchPostInstallHandlerWebroot::toExecute',
        'API::Patch::PatchPostInstallHandlerWebroot::useNow',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_basic',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_universal',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::toExecute',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite::useNow',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_basic',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_universal',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::toExecute',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges::useNow',
        'API::Patch::PatchPostInstallHandlerConfigFiles::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandlerConfigFiles::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_basic',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_universal',
        'API::Patch::PatchPostInstallHandlerConfigFiles::toExecute',
        'API::Patch::PatchPostInstallHandlerConfigFiles::useNow',
        'API::Patch::PatchPostInstallHandlerServices::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandlerServices::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandlerServices::doPatch',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_basic',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandlerServices::doPatch_universal',
        'API::Patch::PatchPostInstallHandlerServices::toExecute',
        'API::Patch::PatchPostInstallHandlerServices::useNow',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::callSubHandlerWithVersionSources',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_1_1',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_1_2',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_1_3',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_2_1',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_2_2',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_2_3',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_2_1_1',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_2_1_2',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_2_1_3',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_basic',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_checkPreCondition',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_universal',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::toExecute',
        'API::Patch::PatchPostInstallHandlerRemoteSearch::useNow'
    ];

    #print Dumper($PatchSystemInfoHandler->validFunctions());

    ok (Dumper($PatchSystemInfoHandler->validFunctions()) eq Dumper($Functions), 'functions correct');

    ok (not ($PatchSystemInfoHandler->isLastVersion('1.2.2')), 'version \'1.2.2\' is not last version');

    ok ($PatchSystemInfoHandler->isLastVersion('2.1.3'), 'version \'2.1.3\' is last version');

    ok (not ($PatchSystemInfoHandler->isFirstVersion('1.2.2')), 'version \'1.2.2\' is not first version');

    ok ($PatchSystemInfoHandler->isFirstVersion('1.1.1'), 'version \'1.1.1\' is first version');

    ok (($PatchSystemInfoHandler->firstVersion() eq '1.1.1'), 'first version eq \'1.1.1\'');

    ok (($PatchSystemInfoHandler->lastVersion() eq '2.1.3'), 'last version eq \'2.1.3\'');

    ok (($PatchSystemInfoHandler->nextVersion('1.2.3') eq '2.1.1'), 'next version from \'1.2.3\' is version \'2.1.1\'');

    eval{$PatchSystemInfoHandler->nextVersion('2.1.3')};
    ok( ExistsError(), 'next version from last version throws an error');

    is ($PatchSystemInfoHandler->previousVersion('2.1.1'), '1.2.3', 'previous version from \'2.1.1\' is version \'1.2.3\'');

    eval{$PatchSystemInfoHandler->previousVersion('1.1.1')};
    ok( ExistsError(), 'previous version from first version throws an error');

    #TO DO validGroundFunction()

    ok (not ($PatchSystemInfoHandler->isPatchFunctionStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' not started for version \'2.1.1\'');

    $PatchSystemInfoHandler->registerStartedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');

    ok ($PatchSystemInfoHandler->isPatchFunctionStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch'), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' now started for version \'2.1.1\'');

    ok (not ($PatchSystemInfoHandler->isPatchFunctionCompleted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' not completed for version \'2.1.1\'');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');

    ok ($PatchSystemInfoHandler->isPatchFunctionCompleted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch'), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' now completed for version \'2.1.1\'');

    is ($PatchSystemInfoHandler->getCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir'), undef, 'handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' has no current version');

    $PatchSystemInfoHandler->setCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', '2.1.1');

    is ($PatchSystemInfoHandler->getCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir'), '2.1.1', 'handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' has now current version \'2.1.1\'');

    ok ($PatchSystemInfoHandler->versionLowerThan('1.1.3', '2.1.2'), 'version \'1.1.3\' is lower than version \'2.1.2\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('2.1.2', '1.1.3')), 'version \'2.1.2\' is not lower than version \'1.1.3\'');

    ok ($PatchSystemInfoHandler->versionLowerThan('1.1.2.1', '1.1.3'), 'not existing version \'1.1.2.1\' is lower than existing version \'1.1.3\'');

    ok ($PatchSystemInfoHandler->versionLowerThan('1.1.2.1', '1.1.2.2'), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.2.2\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.3.1', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.3.1\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.3', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than existing version \'1.1.3\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.2.2', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.2.2\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.3.1', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.3.1\'');

    ok ($PatchSystemInfoHandler->versionGreaterThan('2.1.2', '1.1.3'), 'version \'2.1.2\' is greater than version \'1.1.3\'');

    ok (not ($PatchSystemInfoHandler->versionGreaterThan('1.1.3', '2.1.2')), 'version \'1.1.3\' is not greater than version \'2.1.2\'');

    eval{$PatchSystemInfoHandler->versionGreaterThan('6.6.6', '1.1.3')};
    ok( ExistsError(), 'check invalid number throws error');

    eval{$PatchSystemInfoHandler->versionGreaterThan('1.1.3', '6.6.6')};
    ok( ExistsError(), 'check invalid number throws error');


    #tests for cartridges version folder
    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does not exist for version \'1.1.3\'');

    eval{$PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3')};
    ok( ExistsError(), 'get version dir without initialization throws error');

    $PatchSystemInfoHandler->setBasicAttribute('EpagesCartridgesVersionDir','C:\epages\Cartridges');

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3'), 'C:/epages/Cartridges/1.1.3', 'cartridges version folder for version \'1.1.3\' correct \'C:/epages/Cartridges/1.1.3\'');

    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does not exist for version \'1.1.3\' without set manualy');

    $PatchSystemInfoHandler->setCartridgesVersionFolder('1.1.3', 'D:/elsewhere/Test/1.1.3');

    ok (($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does now exists exist for version \'1.1.3\'');

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3'), 'D:/elsewhere/Test/1.1.3', 'cartridges version folder for version \'1.1.3\' correct changed \'D:/elsewhere/Test/1.1.3\'');

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('2.1.1'), 'C:/epages/Cartridges/2.1.1', 'cartridges version folder for version \'2.1.1\' stay correct by default \'C:/epages/Cartridges/2.1.1\'');

    $PatchSystemInfoHandler->setCartridgesVersionFolder('1.1.3', 'C:/epages/Cartridges/1.1.3');

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3'), 'C:/epages/Cartridges/1.1.3', 'cartridges version folder for version \'1.1.3\' correct setting back to \'C:/epages/Cartridges/1.1.3\'');


    #tests for patch cartridges version folder
    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','PatchCartridgesVersionFolder')), 'version attribute \'PatchCartridgesVersionFolder\' does not exist for version \'1.1.3\'');

    eval{$PatchSystemInfoHandler->getPatchCartridgesVersionFolder('1.1.3')};
    ok( ExistsError(), 'get version dir without initialization throws error');

    $PatchSystemInfoHandler->setBasicAttribute('PatchCartridgesVersionDir','C:\Temp\patch_x_y_z\Cartridges.v');

    is ( $PatchSystemInfoHandler->getPatchCartridgesVersionFolder('1.1.3'), 'C:/Temp/patch_x_y_z/Cartridges.v/1.1.3', 'patch cartridges version folder for version \'1.1.3\' correct \'C:/Temp/patch_x_y_z/Cartridges.v/1.1.3\'');

    is ( $PatchSystemInfoHandler->getPatchCartridgesVersionFolder('2.1.1'), 'C:/Temp/patch_x_y_z/Cartridges.v/2.1.1', 'patch cartridges version folder for version \'2.1.1\' correct by default \'C:/Temp/patch_x_y_z/Cartridges.v/2.1.1\'');

    $PatchSystemInfoHandler->setBasicAttribute('PatchWebrootVersionDir', 'C:/Temp/patch_x_y_z/Shared/Webroot.v');

    #getPatchWebrootVersionFolder
    is ( $PatchSystemInfoHandler->getPatchWebrootVersionFolder('1.1.3'), 'C:/Temp/patch_x_y_z/Shared/Webroot.v/1.1.3', 'patch webroot version folder for version \'1.1.3\' correct \'C:/Temp/patch_x_y_z/Shared/Webroot.v/1.1.3\'');


    #test storetypes
    ok (not ($PatchSystemInfoHandler->existsStoreType('StoreType1')), 'store type \'StoreType1\' does not exist');

    $PatchSystemInfoHandler->addStoreType('StoreType1');

    ok ($PatchSystemInfoHandler->existsStoreType('StoreType1'), 'store type \'StoreType1\' now exist');

    $PatchSystemInfoHandler->addStoreType('StoreType2');
    $PatchSystemInfoHandler->addStoreType('StoreType6');
    $PatchSystemInfoHandler->addStoreType('StoreType7');
    $PatchSystemInfoHandler->addStoreType('StoreType5');
    $PatchSystemInfoHandler->addStoreType('StoreType3');
    $PatchSystemInfoHandler->addStoreType('StoreType4');
    $PatchSystemInfoHandler->addStoreType('AAA');

    my $aStoreTypes = [
      'AAA',
      'StoreType1',
      'StoreType2',
      'StoreType3',
      'StoreType4',
      'StoreType5',
      'StoreType6',
      'StoreType7'
    ];
    is_deeply( $PatchSystemInfoHandler->getStoreTypes(), $aStoreTypes, 'store types correct');

    ok(not($PatchSystemInfoHandler->existsStoreTypeCartridgeVersion('1.1.1','StoreType1','DE_EPAGES::CartridgeXY')), 'cartridge \'DE_EPAGES::CartridgeXY\' dont exists in store type \'StoreType1\'');

    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXY', 7);

    ok($PatchSystemInfoHandler->existsStoreTypeCartridgeVersion('1.1.1','StoreType1','DE_EPAGES::CartridgeXY'), 'cartridge \'DE_EPAGES::CartridgeXY\' now exists in store type \'StoreType1\' for version \'1.1.1\'');

    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXB', 3);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXG', 4);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXH', 5);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXZ', 8);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXA', 2);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXK', 6);

    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType2', 'DE_EPAGES::CartridgeAZ', 1);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType3', 'DE_EPAGES::CartridgeZZ', 1);

    $PatchSystemInfoHandler->addStore('TestStore1','StoreType1');
    $PatchSystemInfoHandler->addStore('TestStore2','StoreType1');
    $PatchSystemInfoHandler->addStore('TestStore3','StoreType1');

    $PatchSystemInfoHandler->addStore('TestStore4','StoreType2');
    $PatchSystemInfoHandler->addStore('TestStore5','StoreType2');
    $PatchSystemInfoHandler->addStore('TestStore6','StoreType2');

    ok($PatchSystemInfoHandler->isCartridgeInOneStore('DE_EPAGES::CartridgeXA'), 'cartridge \'DE_EPAGES::CartridgeXA\' is in a store');
    ok($PatchSystemInfoHandler->isCartridgeInOneStore('DE_EPAGES::CartridgeAZ'), 'cartridge \'DE_EPAGES::CartridgeAZ\' is in a store');
    ok(not($PatchSystemInfoHandler->isCartridgeInOneStore('DE_EPAGES::CartridgeZZ')),'cartridge \'DE_EPAGES::CartridgeZZ\' is not in a store');

    $PatchSystemInfoHandler->addStore('AAA','StoreType1');
    $PatchSystemInfoHandler->addStore('BBB','StoreType2');

    ok( $PatchSystemInfoHandler->isCartridgeInStore('TestStore1', 'DE_EPAGES::CartridgeXB') , 'cartridge \'DE_EPAGES::CartridgeXB\' is in store \'TestStore1\'');

    ok( not ($PatchSystemInfoHandler->isCartridgeInStore('TestStore1', 'DE_EPAGES::CartridgeAZ')) , 'cartridge \'DE_EPAGES::CartridgeAZ\' is not in store \'TestStore1\'');

    my $aAllCartridges = [
        'DE_EPAGES::CartridgeAZ',
        'DE_EPAGES::CartridgeXA',
        'DE_EPAGES::CartridgeXB',
        'DE_EPAGES::CartridgeXG',
        'DE_EPAGES::CartridgeXH',
        'DE_EPAGES::CartridgeXK',
        'DE_EPAGES::CartridgeXY',
        'DE_EPAGES::CartridgeXZ',
        'DE_EPAGES::CartridgeZZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getAllCartridges(), $aAllCartridges, 'all cartridges correct');

    ok(($PatchSystemInfoHandler->isCartridgeUsingStandardVersions('DE_EPAGES::CartridgeAZ')), 'cartridge use standard versions');

    $PatchSystemInfoHandler->setCartridgeUseStandardVersions('DE_EPAGES::CartridgeAZ', 0);

    ok(not ($PatchSystemInfoHandler->isCartridgeUsingStandardVersions('DE_EPAGES::CartridgeAZ')), 'cartridge use standard versions');

    $PatchSystemInfoHandler->setCartridgeUseStandardVersions('DE_EPAGES::CartridgeAZ', 1);


    my $aCartridgesInVersion = [
        'DE_EPAGES::CartridgeAZ',
        'DE_EPAGES::CartridgeXA',
        'DE_EPAGES::CartridgeXB',
        'DE_EPAGES::CartridgeXG',
        'DE_EPAGES::CartridgeXH',
        'DE_EPAGES::CartridgeXK',
        'DE_EPAGES::CartridgeXY',
        'DE_EPAGES::CartridgeXZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getCartridgesInVersion('1.1.1'), $aCartridgesInVersion, 'correct cartridges in version \'1.1.1\'');

    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXH', 1);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXZ', 2);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXA', 3);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXK', 4);

    $aCartridgesInVersion = [
        'DE_EPAGES::CartridgeXA',
        'DE_EPAGES::CartridgeXH',
        'DE_EPAGES::CartridgeXK',
        'DE_EPAGES::CartridgeXZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getCartridgesInVersion('1.1.2'), $aCartridgesInVersion, 'correct cartridges in version \'1.1.2\'');

    #stores
    ok( $PatchSystemInfoHandler->existsStore('TestStore1') , 'Store \'TestStore1\' exists');
    ok( not ($PatchSystemInfoHandler->existsStore('TestStoreXY')) , 'Store \'TestStore2\' not exists');

    my $aStores = [
        'AAA',
        'BBB',
        'TestStore1',
        'TestStore2',
        'TestStore3',
        'TestStore4',
        'TestStore5',
        'TestStore6',
    ];
    is_deeply( $PatchSystemInfoHandler->getStores(), $aStores, 'stores correct');

    $aStores = [
        'AAA',
        'TestStore1',
        'TestStore2',
        'TestStore3',
    ];
    is_deeply( $PatchSystemInfoHandler->getStoreTypeStores('StoreType1'), $aStores, 'stores corrects for \'StoreType1\'');

    $aStores = [
        'BBB',
        'TestStore4',
        'TestStore5',
        'TestStore6',
    ];
    is_deeply( $PatchSystemInfoHandler->getStoreTypeStores('StoreType2'), $aStores, 'stores corrects for \'StoreType2\'');

    is($PatchSystemInfoHandler->getStoreStoreType('AAA'), 'StoreType1', 'store have correct store type');
    is($PatchSystemInfoHandler->getStoreStoreType('BBB'), 'StoreType2', 'store have correct store type');

    ok(not ($PatchSystemInfoHandler->isStoreStarted('1.1.1','AAA')), 'store is not started to patch yet');
    ok(not ($PatchSystemInfoHandler->isStoreInProcess('AAA')), 'store is not in process yet');

    $PatchSystemInfoHandler->registerStartedStore('1.1.1','AAA', '1234567890');

    ok(not ($PatchSystemInfoHandler->allStoreStoresCompleted('1.1.1')), 'not all stores are completed');

    ok($PatchSystemInfoHandler->getProcessIDForStartedStore('AAA') eq '1234567890', 'correct processid');
    ok($PatchSystemInfoHandler->isStoreInProcess('AAA'), 'store is now in process');
    ok($PatchSystemInfoHandler->isStoreStarted('1.1.1','AAA'), 'store is now started to patch');
    ok(not ($PatchSystemInfoHandler->isStoreCompleted('1.1.1','AAA')), 'store is not complete patched yet');
    ok($PatchSystemInfoHandler->existsStoreProcessByStore('AAA'), 'process exists for started store');

    eval{$PatchSystemInfoHandler->registerCompletedStore('1.1.1','AAA')};
    ok( ExistsError(), 'cant register completed store because of unfinished cartridges');

    ok(not ($PatchSystemInfoHandler->isStoreCartridgeStarted('1.1.1','AAA','DE_EPAGES::CartridgeXA')), 'store cartridge is not started');
    ok(not ($PatchSystemInfoHandler->isStoreCartridgeCompleted('1.1.1','AAA','DE_EPAGES::CartridgeXA')), 'store cartridge is not completed');

    eval{$PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXA')};
    ok( ExistsError(), 'cant register completed store cartridge because it is not started yet');

    #started store cartridges
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXZ');

    eval{$PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeAZ')};
    ok( ExistsError(), 'cant register completed store cartridge because doesnt contain the cartrige');

    ok($PatchSystemInfoHandler->isStoreCartridgeStarted('1.1.1','AAA','DE_EPAGES::CartridgeXA'), 'store cartridge is now started');


    #complete cartridges
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXA');

    ok($PatchSystemInfoHandler->isStoreCartridgeCompleted('1.1.1','AAA','DE_EPAGES::CartridgeXA'), 'store cartridge is now completed');

    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXY');

    ok(not ($PatchSystemInfoHandler->allStoreCartridgesCompleted('1.1.1','AAA')), 'not all store cartridges are completed');

    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXZ');

    ok($PatchSystemInfoHandler->allStoreCartridgesCompleted('1.1.1','AAA'), 'now all store cartridges are completed');

    #store
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','AAA');

    #BBB
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','BBB', '1234567890');
    ok($PatchSystemInfoHandler->getProcessIDForStartedStore('BBB') eq '1234567890', 'correct processid');
    ok($PatchSystemInfoHandler->isStoreInProcess('BBB'), 'store is now in process with already used processid');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','BBB','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','BBB','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','BBB');

    #TestSTore1
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore1', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore1');

    #TestStore2
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore2', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore2');

    #TestStore3
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore3', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore3');

    #TestStore4
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore4', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore4','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore4','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore4');

    #TestStore5
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore5', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore5','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore5','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore5');

    #TestStore6
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore6', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore6','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore6','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore6');


    ok($PatchSystemInfoHandler->allStoreStoresCompleted('1.1.1'), 'all stores are completed');

    ok($PatchSystemInfoHandler->isStoreCompleted('1.1.1','AAA'), 'store is now completed with patch');
    ok(not ($PatchSystemInfoHandler->isStoreInProcess('AAA')), 'store is no longer in process');
    ok(not ($PatchSystemInfoHandler->existsCurrentStoreVersion('AAA')), 'no store version exists yet');
    $PatchSystemInfoHandler->setCurrentStoreVersion('AAA','1.1.1');
    ok($PatchSystemInfoHandler->existsCurrentStoreVersion('AAA'), 'store version exists now');
    ok(($PatchSystemInfoHandler->getCurrentStoreVersion('AAA') eq '1.1.1'), 'store has correct version');

    #test cartridges not using standard version numbers
    $PatchSystemInfoHandler->registerStartedStore('1.1.2','TestStore3', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXY');

    eval{
        $PatchSystemInfoHandler->registerCompletedStore('1.1.2','TestStore3');
    };
    ok( ExistsError(), 'correct error for complete store');

    $PatchSystemInfoHandler->setCartridgeUseStandardVersions('DE_EPAGES::CartridgeXZ', 0);

    eval{
        $PatchSystemInfoHandler->registerCompletedStore('1.1.2','TestStore3');
    };
    ok( not (ExistsError()), 'correct to complete store, ignore cartridges that dont using standard version numbers');

    #cartridges
    my $aCartridges = [
      'DE_EPAGES::CartridgeXA',
      'DE_EPAGES::CartridgeXB',
      'DE_EPAGES::CartridgeXG',
      'DE_EPAGES::CartridgeXH',
      'DE_EPAGES::CartridgeXK',
      'DE_EPAGES::CartridgeXY',
      'DE_EPAGES::CartridgeXZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getStoreTypeCartridges('1.1.1','StoreType1'), $aCartridges, 'store type cartridges correct for store type \'StoreType1\'');
    is_deeply( $PatchSystemInfoHandler->getStoreCartridges('1.1.1','AAA'), $aCartridges, 'store cartridges correct for store \'AAA\'');
    is($PatchSystemInfoHandler->getStoreStoreType('AAA'), 'StoreType1', 'store type correct for store \'AAA\'');


    #started patch function
    is(scalar @{$PatchSystemInfoHandler->getStartedPatchFunctions('1.1.1')}, 0, 'no strted patch function for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::ANYTHING::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid handler throws error');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_ANYTHING')};
    ok( ExistsError(), 'register patch funtion with invalid ground function name throws error');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('9.9.9', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid version throws error');

    my $aStartedPatchFunctions = [
        'API::Patch::PatchInstallHandlerStores::doPatch',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerStores::doPatch_basic',
        'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal'
    ];

    is_deeply( $PatchSystemInfoHandler->getStartedPatchFunctions('1.1.1'), $aStartedPatchFunctions, 'started patch functions correct for version \'1.1.1\'');


    #completed patch functions
    is(scalar @{$PatchSystemInfoHandler->getCompletedPatchFunctions('1.1.1')}, 0, 'no completed patch function for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::ANYTHING::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid handler throws error');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_ANYTHING')};
    ok( ExistsError(), 'register patch funtion with invalid ground function name throws error');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('9.9.9', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid version throws error');

    my $aCompletedPatchFunctions = [
        'API::Patch::PatchInstallHandlerStores::doPatch',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerStores::doPatch_basic',
        'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal'
    ];

    is_deeply( $PatchSystemInfoHandler->getCompletedPatchFunctions('1.1.1'), $aCompletedPatchFunctions, 'completed patch functions correct for version \'1.1.1\'');

    #log level tests
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch') == 4, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('callSubHandlerWithVersionSources') == 3, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('toExecute') == 2, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_checkPreCondition') == 2, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_basic') == 1, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_universal') == 0, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_2_1_3') == 0, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('useNow') == 0, 'correct loglevel');

    #start time
    ok($StartTime eq $PatchSystemInfoHandler->getStartTime(), 'correct start time');

    #skip
    ok(not ($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler')), "correct skip value");

    $PatchSystemInfoHandler->setHandlerSkip('API::Patch::PatchPrepareHandler', 1);

    ok($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler'), "correct skip value");

    $PatchSystemInfoHandler->setHandlerSkip('API::Patch::PatchPrepareHandler', 0);

    ok(not ($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler')), "correct skip value");

    ok(not ($PatchSystemInfoHandler->existsLanguage('en-US')), "language not exists");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en-US')), "language not exists in store for version");

    $PatchSystemInfoHandler->addLanguage('en');

    ok($PatchSystemInfoHandler->existsLanguage('en'), "language exists");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en')), "language not exists in store for version");

    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'en');

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.2','TestStore1','en')), "language not exists in store for version");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore2','en')), "language not exists in store for version");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en-US')), "language not exists in store for version");

    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en'), "language exists in store for version");

    my $aLanguages;

    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'en-US');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en-US'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'fr');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','fr'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'fi');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','fi'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'da');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','da'), "language exists in store for version");

    $aLanguages = ['da','en','en-US','fi','fr'];

    is_deeply( $PatchSystemInfoHandler->getStoreLanguages('1.1.1', 'TestStore1'), $aLanguages, 'correct languages');


    $PatchSystemInfoHandler->addLanguageToStore('1.1.2', 'TestStore1', 'es');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.2','TestStore1','es'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.2', 'TestStore1', 'no');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.2','TestStore1','no'), "language exists in store for version");

    $aLanguages = ['es', 'no'];

    is_deeply( $PatchSystemInfoHandler->getStoreLanguages('1.1.2', 'TestStore1'), $aLanguages, 'correct languages');

    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore2', 'it');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore2','it'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore2', 'ru');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore2','ru'), "language exists in store for version");

    $aLanguages = ['it', 'ru'];

    is_deeply( $PatchSystemInfoHandler->getStoreLanguages('1.1.1', 'TestStore2'), $aLanguages, 'correct languages');

    my $EpagesSharedPatchesDir = RealPath($ENV{EPAGES_SHARED}). '/Patches';
    $PatchSystemInfoHandler->setBasicAttribute('EpagesSharedPatchesDir', $EpagesSharedPatchesDir) unless $PatchSystemInfoHandler->existsBasicAttribute('EpagesSharedPatchesDir');
    ok($PatchSystemInfoHandler->getCartridgesDataPackageFile('1.1.1') =~ /\/patch_1\.1\.1\/epages-1\.1\.1-patch-data\.tar\.gz/ , "cartridges data package correct for version '1.1.1'");

    Cleanup($CurrentDir);
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [flags]

flags:
    -help          show the command line options

Example:
    perl $0
END_USAGE
    exit 2;
}

RunScript( Message => 'long', Sub => \&Main);

1;
